@@ -20,13 +20,23 @@
 #define _ITER_H_
 #include <core.h>
 namespace Iter {
-  struct Iter<`a::A,`eff::E> { 
-    <`env>
+  /*** \subsection{\texttt{<iter.h>}} */
+  /*** Defines namespace Iter, which implements imperative iterators
+       over sets/sequences of elements.
+  */
+
+  struct Iter<`a::A,`bd::R> { 
+    <`env> : regions(`env) > `bd
     `env env;
     // return false when no more elts.
-    bool (@next)<`r::R>(`env env, `a @`r dest; `eff + {`r}); 
+    bool (@next)<`r>(`env env, `a @`r dest); 
   };
-  typedef struct Iter<`a,`eff> iter_t<`a,`eff>;
-  bool next(iter_t<`a::A>,`a::A @);
+  typedef struct Iter<`a,`bd> iter_t<`a,`bd>;
+  /** A value of type [iter_t<`a,`bd>] is an iterator over elements
+      of type [`a]. */
+  bool next(iter_t<`a::A>,`a @);
+  /** If there is a next element, [next(i,p)] returns [true] and
+      assigns the next element to [*p].  If there is no next element,
+      [next(i,p)] returns [false] without assigning anything to [*p]. */
 }
 #endif
