@@ -28,19 +28,25 @@ namespace List {
        various operations over them, following the conventions of the
        Objective Caml list library as much as possible. */
 
-  struct List<`a,`r::R>{`a hd; struct List<`a,`r> *`r tl;};
+  struct List<`a,`r>{`a hd; struct List<`a,`r> *`r tl;};
   /** A [struct List] is a memory cell with a head field containing an
       element and a tail field that points to the rest of the list.
       Such a structure is traditionally called a cons cell.  Note that
       every element of the list must have the same type [`a], and
       every cons cell in the list must be allocated in the same region
       [`r].  */
+
   typedef struct List<`a,`r> *`r list_t<`a,`r>;
   /** A [list_t] is a possibly-NULL pointer to a [struct List].  Most
       of the functions in namespace List operate on values of type
       [list_t] rather than [struct List].  Note that a [list_t] can be
       empty (NULL) but a [struct List] cannot. */
-  typedef struct List<`a,`r> @List_t<`a,`r>;
+
+//   typedef struct List<`a,`r::TR> *`r trlist_t<`a,`r>;
+//   /** A [trlist_t] is the same as a list_t except that the list can
+//       reside in either a normal region or the unique region.  */
+
+  typedef struct List<`a,`r> @`r List_t<`a,`r>;
   /** A [List_t] is a non-NULL pointer to a [struct List].  This is
       used much less often than [list_t], however it may be useful
       when you want to emphasize that a list has at least one element.  */
@@ -52,16 +58,14 @@ namespace List {
   /** [rlist(r, x1,...,xn)] builds a list with elements [x1] through
       [xn], allocated in the region with handle [r]. */
 
-  extern int length(list_t x);
+  extern int length(list_t<`a::TB> x);
   /** [length(x)] returns the number of elements in list [x].  */
 
-  extern `a hd(list_t<`a> x);
-  /** [hd(x)] returns the first element of list [x], if there is one,
-      and throws [Failure("hd")] if [x] is NULL. */
+  extern `a hd(List_t<`a> x);
+  /** [hd(x)] returns the first element of list [x]. */
 
-  extern list_t<`a,`r> tl(list_t<`a,`r> x);
-  /** [tl(x)] returns the tail of list [x], if there is one,
-      and throws [Failure("tl")] if [x] is NULL. */
+  extern list_t<`a,`r> tl(List_t<`a::TB,`r> x);
+  /** [tl(x)] returns the tail of list [x]. */
 
   extern list_t<`a> copy(list_t<`a> x);
   /** [copy(x)] returns a new heap-allocated copy of list [x]. */
@@ -83,7 +87,7 @@ namespace List {
   /** [rmap_c] is a version of [rmap] where the function argument
       requires a closure as its first argument. */
 
-  extern xtunion exn {extern List_mismatch};
+  extern datatype exn {extern List_mismatch};
   /** [List_mismatch] is thrown when two lists don't have the same
       length. */
 
@@ -97,6 +101,18 @@ namespace List {
                                list_t<`a> x,list_t<`b> y);
   /** [rmap2(r,f,x,y)] is like [map2(f,x,y)], except that the
       resulting list is allocated in the region with handle [r]. */
+  extern list_t<`d> map3(`d f(`a,`b,`c),
+                         list_t<`a> x,list_t<`b> y,list_t<`c> z);
+  /** If [x] has elements [x1] through [xn], [y] has elements [y1]
+      through [yn], and [z] has elements [z1] through [zn], then 
+      [map3(f,x,y,z)] returns a new heap-allocated list with elements 
+      [f(x1,y1,z1)] through [f(xn,yn,zn)].  If [x], [y], and
+      [z] don't have the same number of elements, [List_mismatch] is
+      thrown. */
+  extern list_t<`d,`r> rmap3(region_t<`r>,`d f(`a,`b,`c),
+                             list_t<`a> x,list_t<`b> y,list_t<`c> z);
+  /** [rmap3(r,f,x,y,z)] is like [map3(f,x,y,z)], except that the
+      resulting list is allocated in the region with handle [r]. */
 
   extern void app(`b f(`a),list_t<`a> x);
   /** [app(f,x)] applies [f] to each element of [x], discarding the
@@ -159,7 +175,7 @@ namespace List {
   extern list_t<`a,`r> rrev(region_t<`r>,list_t<`a> x);
   /** [rrev(r,x)] is like [rev(x)], except that the result is
       allocated in the region with handle [r].  */
-  extern list_t<`a,`r> imp_rev(list_t<`a,`r> x);
+  extern list_t<`a,`r> imp_rev(list_t<`a::TB,`r> x);
   /** [imp_rev(x)] imperatively reverses list [x] (the list is
       side-effected).  Note that [imp_rev] returns a list.  This is
       because the first cons cell of the result is the last cons cell
@@ -176,7 +192,7 @@ namespace List {
       allocated in the region with handle [r], and the result is
       allocated in the same region.  */
 
-  extern list_t<`a,`r> imp_append(list_t<`a,`r> x,list_t<`a,`r> y);
+  extern list_t<`a,`r> imp_append(list_t<`a::TB,`r> x,list_t<`a,`r> y);
   /** [imp_append(x,y)] modifies [x] to append [y] to it,
       destructively.  Note that [imp_append] returns a list.  This is
       because [x] might be NULL, in which case, [imp_append(x,y)]
@@ -219,7 +235,7 @@ namespace List {
                                  list_t<`a,`r> a,list_t<`a,`r> b);
   /** [imp_merge] is an imperative version of [merge]. */
 
-  extern xtunion exn {extern Nth};
+  extern datatype exn {extern Nth};
   /** [Nth] is thrown when [nth] doesn't have enough elements in the
        list.  */
 
@@ -246,6 +262,11 @@ namespace List {
   /** [exists_c] is a version of [exists] where the function
       argument requires a closure as its first argument. */
 
+  extern `c *`r find_c(`c::TA *`r pred(`a,`b),`a env,list_t<`b> x);
+  /** [find_c] iterates over the given list and returns the first element
+      for which [pred] does not return NULL.  Otherwise it returns
+      NULL. */
+
   extern list_t<$(`a,`b)@`H,`H> zip(list_t<`a> x,list_t<`b> y);
   /** If [x] has elements [x1] through [xn], and [y] has elements [y1]
       through [yn], then [zip(x,y)] returns a new heap-allocated array
@@ -321,11 +342,19 @@ namespace List {
   extern bool mem_assoc(list_t<$(`a,`b)@> l,`a x);
   /** [mem_assoc(l,k)] returns true if [k] is a key in association
       list [l] (according to [==]).  */
+  extern bool mem_assoc_cmp(int cmp(`a,`c),list_t<$(`a,`b)@> l,`c x);
+  /** Same as [mem_assoc], but uses comparison function [cmp] rather
+      than pointer equality ==. */
 
   extern list_t<`a,`r::R> delete(list_t<`a,`r::R> l, `a x);
   /** [delete(l,k)] returns the list with the first occurence of [x]
       removed from it, if [x] was in the list; otherwise raises
-      [Core::Not_found].  */
+      [Core::Not_found].  Side-effects original list [l]. */
+
+  extern list_t<`a,`r::R> delete_cmp(int cmp(`a,`a), list_t<`a,`r::R> l, `a x);
+  /** [delete(l,k)] returns the list with the first [e] in the list
+      such that [cmp]([x],[e]) == 0.  If no such [e] exists, raises
+      [Core::Not_found].  Side-effects original list [l]. */
 
   extern Core::opt_t<`c> check_unique(int cmp(`c,`c),list_t<`c> x);
   /** [check_unique(cmp,x)] checks whether the sorted list [x] has
