@@ -21,7 +21,7 @@
 #define _ABSYNDUMP_H_
 
 #include <core.h>
-#include <stdio.h>
+#include <cycboot.h>
 #include <list.h>
 #include "absyn.h"
 
@@ -29,7 +29,8 @@
 
 namespace Absyndump {
 extern void set_params(struct Absynpp::Params @ `r fs);
-extern void dumpdecllist2file(List::list_t<Absyn::decl_t> tdl, FILE @f);
+extern void dumpdecllist2file(List::list_t<Absyn::decl_t> tdl, FILE @`H f);
+extern void dump_interface(List::list_t<Absyn::decl_t> ds, FILE @`H f);
 }
 
 #endif
