@@ -31,7 +31,9 @@ using Absyn;
 // translate the declarations to C
 extern list_t<decl_t> toc(list_t<decl_t> ds);
 
-  // this is just for Tovc
-  extern qvar_t temp_var();
+// this is just for Tovc and tcStmt
+extern qvar_t temp_var();
+extern stringptr_t ?globals;
+
 }
 #endif
