@@ -20,17 +20,21 @@ SYNTAX_EXTERN_DEFINITION struct Location {
 };
 typedef struct Location @ location_t;
 
-SYNTAX_EXTERN_DEFINITION tunion Regular_expression {
-  Epsilon;
+SYNTAX_EXTERN_DEFINITION datatype Regular_expression {
+  Epsilon(int);
   Characters(list_t<int>);
-  Sequence(tunion Regular_expression, tunion Regular_expression);
-  Alternative(tunion Regular_expression, tunion Regular_expression);
-  Repetition(tunion Regular_expression);
+  Sequence(datatype Regular_expression@, datatype Regular_expression@);
+  Alternative(datatype Regular_expression@, datatype Regular_expression@);
+  Repetition(datatype Regular_expression@);
 };
-typedef tunion Regular_expression regular_expression_t;
+typedef datatype Regular_expression @regular_expression_t;
+extern datatype Regular_expression.Epsilon Epsilon_val;
+#ifdef SYNTAX_CYC
+datatype Regular_expression.Epsilon Epsilon_val = Epsilon(0);
+#endif
 
 typedef $(regular_expression_t,location_t) @ acase_t;
-typedef $(string_t,string_t,list_t<acase_t>) @ entrypoint_t;
+typedef $(string_t,string_t,$(string_t,string_t)*,list_t<acase_t>) @ entrypoint_t;
 
 SYNTAX_EXTERN_DEFINITION struct Lexer_definition {
   location_t         header;
